#====================================================================================
# This script is to test various modes of S10 KR PHY in hardware 
# This script should be run in the System Console window
# Type the command "source $PATH/console.tcl" in the System Console window to execute
#====================================================================================

#==============================================================================
# Some global variables here 
#==============================================================================
global master_claim_path
global MAIN_DICT_s10_10g_kr_0
global freeze1
global freeze2
global accum_send_packets

set accum_send_packets 0

dict set MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR 0x0800
dict set MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO 0x0000

# ================================================================== 
# 	XCVR RECONFIG
# ==================================================================
dict set MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPBLOCK 0x171
dict set MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE 0x149
dict set MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD 0x17E
dict set MAIN_DICT_s10_10g_kr_0 ADDR_LT_VOD 0x109
dict set MAIN_DICT_s10_10g_kr_0 ADDR_LT_POST 0x105
dict set MAIN_DICT_s10_10g_kr_0 ADDR_LT_PRE 0x107

# ================================================================== 
# 	General 40GBASE-KR4/CR4 Registers
# ==================================================================
#
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B0	0x04B0
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B1	0x04B1
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B2	0x04B2

# ================================================================== 
# 	Auto-negotiation Registers
# ==================================================================
#
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C0	0x04C0
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C1	0x04C1
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C2	0x04C2
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C3	0x04C3
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C4	0x04C4
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C5	0x04C5
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C6	0x04C6
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C7	0x04C7
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C8	0x04C8
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C9	0x04C9
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04CA	0x04CA
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04CB	0x04CB

# ================================================================== 
# 	Link training Registers
# ==================================================================
#
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04D0	0x04D0
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04D1	0x04D1
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04D2	0x04D2
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04D3	0x04D3
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04D4	0x04D4
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04D5	0x04D5
dict set MAIN_DICT_s10_10g_kr_0 ADDR_KR_04D6	0x04D6


# Change declaration of ch_all/th_all below if no of channels/test-harness are changed in design example
set ch_all [list 0 1]
set th_all [list 0]

#==============================================================================
# reconfig_read           
# This proc needs address and slave-name is input arguments
# Slave name is determined by proc "set_up_channels"
#==============================================================================
proc reconfig_read { channel address_hex } {
    global master_claim_path
    set address_hex [expr $address_hex | [expr $channel<<12]]	
    set data [master_read_32 $master_claim_path [expr {$address_hex*4}] 1] 
    return $data
}
#==============================================================================
# reconfig_write          
# This proc needs address, data, and slave-name is input arguments
# Slave name is determined by proc "set_up_channels"
#==============================================================================
proc reconfig_write { channel address_hex value} {
    global master_claim_path
    set address_hex [expr $address_hex | [expr $channel<<12]]	
    master_write_32 $master_claim_path [expr {$address_hex*4}] $value 
}

# help proc read-modify-write RMW
proc rdmwr {channel address  bitmask newval} {
    set address [expr $address | [expr $channel<<12]]	
    set value [reconfig_read $channel $address]
    set value [expr $value & [expr 0xFFFFFFFF & ~$bitmask]]
    set value [expr $value | $newval]

    reconfig_write $channel $address $value
}

proc reconfig_reg_write {chan addr offset val} {
        
    set chanoffs [expr $offset + (0x800 * $chan)]
    return [reg_write $chanoffs $addr $val]
}

proc reconfig_reg_read {chan addr offset} {
       
    set chanoffs [expr $offset + (0x800 * $chan)]
    return [reg_read $chanoffs $addr]
}
proc reconfig_reg_rmw {chan addr offset mask val} {
    set value [reconfig_reg_read $chan $addr $offset]
    set value [expr $value & [expr 0xffffffff & ~$mask]] 
    set value [expr $value | $val] 
    reconfig_reg_write $chan $addr $offset $value
}
proc reg_rmw {addr offset mask val} {
    set value [reg_read $addr $offset]
    set value [expr $value & [expr 0xffffffff & ~$mask]] 
    set value [expr $value | $val] 
    reg_write $addr $offset $value
}
#______________________________________________________________________________
#================================================================================
#  create dummy adme_read/write proc- so prbs_testing_suite doesnt have to change
#================================================================================
proc adme_write { channel address_hex value} {
    reconfig_write $channel $address_hex $value	
}

proc adme_read { channel address_hex } {
    reconfig_read $channel $address_hex
}
#==============================================================================
#  Re-start the link without clearing the registers
#  need read-modify-write as reset bit is shared
#==============================================================================
proc rst {channel} {
global ch_all
    set rd_data    [reconfig_read [lindex $ch_all $channel] 0xcb0];
    set wr_data    [expr $rd_data | 1];
    set data_hex   [format "%#x" $wr_data];
    reconfig_write [lindex $ch_all $channel] 0x0cb0 $data_hex;        # seq_reset for channel
}


proc rst_all {} {
global ch_all
set i 0
    foreach ad $ch_all {
        rst $i
	incr i
    }
}

#============= Enable/Disable serial loopback =================================
proc loop_on {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xc61 1
    }
}

proc loop_off {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xc61 0
    }
}	
#==============================================================================
#  Disable the Sequencer AN timeout, may get stuck if AN has errors
#==============================================================================
proc unh_dis_seqan_timer {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xcb0 2
    }
}

proc unh_en_seqan_timer {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xcb0 0
    }
}
#==============================================================================
#  Disable the LT PMA initialize on max_wait_timeout (bit 15 in 0xD0), 
#==============================================================================
proc dis_max_wait_timer {} {
global ch_all
    foreach ad $ch_all {
        rdmwr $ad 0xcd0 0x2 0x2
    }
}	
#==============================================================================
# Force the data mode for the specific channel
# channel = 0,1,2;  mode = 0 for no force, 1 for Gige, 4 for 10G
#==============================================================================
proc seq_force {channel mode} {
global ch_all
    set address     0xcb0
    set value       [expr 0x106 |[expr [expr $mode << 4] + 1]];      # also set seq reset,recal
    set value_hex   [format "%#x" $value];
    reconfig_write  [lindex $ch_all $channel] $address $value_hex;
}

#==============================================================================
#  Disable AN on all channels
#==============================================================================
proc dis_an {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xcc0 0
    }
}

proc dis_nonce {} {
global ch_all
    foreach ad $ch_all {
	rdmwr $ad 0xcc0 0x80 0x80 
    }
}
#==============================================================================
#  Read 10G status
#==============================================================================
proc rd_10g_stat {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "address 0x482 for ch$i == [reconfig_read $ad 0xc82]";
        incr i
    }
}

#==============================================================================
#  FEC Section here -- inject and clear FEC errors
#==============================================================================

#  Clear 10G HSSI error counters 10G/10G-FEC for all channels
proc clr_hssi_err_all { } {
global ch_all
set i 0
    foreach ad $ch_all {
        clr_hssi_err $ad 
        incr i
    }
}

#  Clear 10G HSSI error counters 10G/10G-FEC
proc clr_hssi_err { channel } {
   puts "Clearing error counters"
   set address     0xC81
   reconfig_write $channel $address 0x4 
   reconfig_write $channel $address 0x0 
}

#  Read FEC error counts
proc rd_fec_errors { channel} {
   set add_corr     0xDC
   set corr_err     [reconfig_read $channel $add_corr]
   set add_uncr     0xE0
   set uncr_err     [reconfig_read $channel $add_uncr]
   puts "FEC errors for channel $channel:- Correctable:- $corr_err   Uncorrectable:- $uncr_err"
}

# Read FEC error counts for all channels
proc rd_fec_errors_all {} {
global ch_all
set i 0
    foreach ad $ch_all {
        rd_fec_errors $i
        incr i
    }
}


#  Inject 1 FEC errors for all burst lenghts for all channels
proc inj_traffic_err_all { } {
global ch_all
set i 0
    foreach ad $ch_all {
        inj_traffic_err $i
        incr i
    }
}

#  Inject 1 FEC errors for all burst lenghts
proc inj_traffic_err { channel } {
set x 0
   puts "Inject 1 error- across all burst lenghts for channel $channel"
   set add_bl       0xBD
   set add_inj      0xCB2
   for {set x 0} {$x < 16} {incr x} {
      reconfig_write $channel $add_bl  [expr 0x0E & $x*16]
      reconfig_write $channel $add_inj 0x800
      # Wait for 1 ms before inserting another error
      # This will make sure errors are inserted in different FEC frames
      after 1
   }
}



#==============================================================================
#  Read status for all channels
#==============================================================================
# Sequencer status registers 0xB0 and 0xB1
proc rd_seq_stat {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "Channel:$i  Address 0x4B0 == [reconfig_read $ad 0xcb0]";
        puts "           Address 0x4B1 == [reconfig_read $ad 0xcb1]";
        incr i
    }
}

# Auto-Neg status registers 0xC0 and 0xC2
proc rd_an_stat {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "Channel:$i Address 0x4C0 == [reconfig_read $ad 0xCc0]";
        puts "          Address 0x4C2 == [reconfig_read $ad 0xCc2]";
        incr i
    }
}

# link training status registers 0xD0 and 0xD2
proc rd_lt_stat {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "Channel:$i Address 0xcD0 == [reconfig_read $ad 0xcd0]";
        puts "          Address 0xcD2 == [reconfig_read $ad 0xcd2]";
        incr i
    }
}

# link training tap values
# Read PRE/POST/VOD values for all channel from KR IP CSRs
proc rd_lt_taps_ip {} {
global ch_all
set channel 0
    foreach ad $ch_all {
        set rd_data  [reconfig_read $ad 0xcd5];
        set vod      [expr $rd_data & 0x1F]
        set post     [expr $rd_data & 0x3F00]
        set post     [expr $post >> 8]
        set pre      [expr $rd_data & 0XF0000]
        set pre      [expr $pre >> 16]
        set ctle     [expr $rd_data & 0XF00000]
        set ctle     [expr $pre >> 20]
        set ctle_md  [expr $rd_data & 0X3000000]
        set ctle_md  [expr $pre >> 24]
        set dfe      [expr $rd_data & 0XC000000]
        set dfe      [expr $pre >> 26]
        puts "KRIP CSRs for Channel-$channel === VOD:-$vod  POST:-$post  PRE:-$pre"
        incr channel
    }
}

# Read PRE/POST/VOD values for all channel from HSSI DPRIO
proc rd_lt_taps_hssi_all {} {
global ch_all
    foreach ad $ch_all {
    rd_lt_taps_hssi $ad 
    }
}

# Read PRE/POST/VOD values for given channel from HSSI DPRIO
# Post process and print
proc rd_lt_taps_hssi {channel} {
   set post [rd_post_tap $channel] 
   set pre  [rd_pre_tap  $channel] 
   set vod  [rd_vod_tap  $channel] 
   puts "HSSI DPRIO for Channel-$channel === VOD:-$vod  POST:-$post  PRE:-$pre"
}

proc rd_post_tap {slave} {
   set add       0x105
   set rd_data   [reconfig_read $slave $add];
   set rd_data  [expr $rd_data & 0x3F]
   return $rd_data
}   

proc rd_pre_tap {slave} {
   set add      0x107
   set rd_data  [reconfig_read $slave $add];
   set rd_data  [expr $rd_data & 0x1F]
   return $rd_data
}

proc rd_vod_tap {slave} {
   set add       0x109
   set rd_data    [reconfig_read $slave $add];
   set rd_data  [expr $rd_data & 0x1F]
   return $rd_data
}

#==============================================================================
# KRIP related proc here 
#==============================================================================
# link training error accumulation time
proc rd_lt_bertime {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "address 0x4D3 for channel$i== [reconfig_read $ad 0xcD3]";
        incr i
    }
}

proc rd_all_stat {} {
  rd_seq_stat
  rd_an_stat
  rd_lt_stat
  rd_lt_bertime
  rd_lt_taps_ip
  rd_lt_taps_hssi_all
}

#==============================================================================
#  Test harness proceduers - start test/read status etc
#==============================================================================
# start 10G packets test for th0
proc start_xgmii_th0 {} {
global th_all
  reconfig_write 0 0xF000 0x01
}

## start 10G packets test for th2
#proc start_xgmii_th2 {} {
#global th_all
#  reconfig_write 0 0xE000 0x01
#}

# Read non idle data sent during the test- expected value is 418
proc rd_pkt_cnt_th0 {} {
global th_all
  set cnt [reconfig_read 0 0xF004]
  puts "Non-idle count for Channel-0 is :- $cnt"  
  set cnt [reconfig_read 0 0xF005]
  puts "Non-idle count for Channel-1 is :- $cnt"  
}

## Read non idle data sent during the test- expected value is 418
#proc rd_pkt_cnt_th2 {} {
#global th_all
#  set cnt [reconfig_read 0 0xE004]
#  puts "Non-idle count for Channel-2 is :- $cnt"  
#  set cnt [reconfig_read 0 0xE005]
#  puts "Non-idle count for Channel-3 is :- $cnt" 
#}

#==============================================================================
#  Help procedures here
#==============================================================================
proc de_script_intro {} {
  #puts "-----------------------------------------------------------------------------------------"
  #puts "This is a PCS-only PHY IP design for testing Stratix-10 10G-KR PHY"
  #puts "Auto-Negotiation (Clause 73),Link Training (Clause 72) and FEC (Clause 74) are supported"
  #puts "-----------------------------------------------------------------------------------------"
}

proc de_script_help {} {
  #puts " "
  #puts "Channel 0 == address 0x0Cnn == GXBR_4C_RX0/TX0 SMAs on the Si Dev kit"
  #puts "Channel 1 == address 0x1Cnn == GXBR_4C_RX1/TX1 SMAs on the Si Dev kit"
  #puts " "
  #puts "Test Harness 0 == address 0xFnnn"
  #puts " "
  #puts "Use the \" reconfig_write 0 address  data\" command to write a register in Channel-0"
  #puts "Use the \" reconfig_read  0 address\"       command to read a register in Channel-0 - address 0x4nn"
  #puts "Use the \" reconfig_read  1 address\"       command to read a register in Channel1  - address 0x4nn"
  #puts "Use the \" reconfig_write 0 address  data\" command to write a register in Test-hardness-0"
  #puts "Use the \" reconfig_read  0 address\"       command to read a register in Test-hardness-0 - address 0xFnnn"
  #puts "The address and data need to be hexadecimal i.e. 0xnnn where n=0-f"
  #puts " "
  #puts "Use the \" rst_all\"   command to Re-start the link"
  #puts " "
}

proc proc_continuous_read {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	global rs_enabled
	
	freeze_buttons
	while {[string match [dashboard_get_property $dash continuous_read checked] "true"]} {
	proc_read_kr4_status
	if {$rs_enabled} {
	proc_read_fec_status
	}
	if { [dashboard_get_property $dash enable_design_example checked] } {
	proc_start_xgmii_packets
	}
	}
	unfreeeze_buttoms
}

proc freeze_buttons {} {
	global dash
	global freeze1
	global freeze2
	global rs_enabled
	
	dashboard_set_property $dash read_kr4_status enabled false
	dashboard_set_property $dash enable_serial_loopback enabled false
	dashboard_set_property $dash reset_seq enabled false
	dashboard_set_property $dash disable_lf_timer enabled false
	dashboard_set_property $dash seq_force enabled false
	dashboard_set_property $dash enable_tx_tuning enabled false
	dashboard_set_property $dash ignore_tx_nonce enabled false
	dashboard_set_property $dash enable_design_example enabled false
	dashboard_set_property $dash htile_prod enabled false
		
	set freeze1 [dashboard_get_property $dash start_xgmii_packets enabled]
	set freeze2 [dashboard_get_property $dash clear_xgmii_packets enabled]
	dashboard_set_property $dash trigger_adapt_button enabled false
	if {$rs_enabled} {
	dashboard_set_property $dash read_fec_status enabled false
	}
	
	dashboard_set_property $dash start_xgmii_packets enabled false
	dashboard_set_property $dash clear_xgmii_packets enabled false
	dashboard_set_property $dash fec_insert_error enabled false
	
}

proc unfreeeze_buttoms {} {
	global dash
	global freeze1
	global freeze2
	global rs_enabled
	
	dashboard_set_property $dash read_kr4_status enabled true
	dashboard_set_property $dash enable_serial_loopback enabled true
	dashboard_set_property $dash reset_seq enabled true
	dashboard_set_property $dash disable_lf_timer enabled true
	dashboard_set_property $dash seq_force enabled true
	dashboard_set_property $dash enable_tx_tuning enabled true
	dashboard_set_property $dash ignore_tx_nonce enabled true
	dashboard_set_property $dash enable_design_example enabled true
	dashboard_set_property $dash start_xgmii_packets enabled $freeze1
	dashboard_set_property $dash clear_xgmii_packets enabled $freeze2
	dashboard_set_property $dash fec_insert_error enabled true
	dashboard_set_property $dash trigger_adapt_button enabled true
	dashboard_set_property $dash htile_prod enabled true
	
	if {$rs_enabled} {
	dashboard_set_property $dash read_fec_status enabled true
	}
}

proc proc_htile_prod_enable {} {
	
global dash
global htile_prod_enabled 

	if { [dashboard_get_property $dash htile_prod checked] } {
		set htile_prod_enabled 1
		
	} else {
		set htile_prod_enabled 0
		
	}

}

proc proc_read_kr4_status {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	
	proc_read_kr4_status_1
	after 50
	#proc_read_kr4_status_1
	
}	

proc proc_read_kr4_status_1 {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	
	set DATA_KR_04B0 [reg_read [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR]]
	#puts "0x04B0 $DATA_KR_04B0"
		if {[expr $DATA_KR_04B0 & 0x00010000]} {
             dashboard_set_property $dash fec_ena_data color "green"
           } else {
             dashboard_set_property $dash fec_ena_data color "red"
           }
		if {[expr $DATA_KR_04B0 & 0x00040000]} {
             dashboard_set_property $dash fec_req_data color "green"
           } else {
             dashboard_set_property $dash fec_req_data color "red"
           }
	
	
	
	set DATA_KR_04B1 [reg_read [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B1] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] ]
	#puts "0x04B1 $DATA_KR_04B1"
	    if {[expr $DATA_KR_04B1 & 0x00000001]} {
             dashboard_set_property $dash SEQ_LNK_RDY_LED color "green"
           } else {
             dashboard_set_property $dash SEQ_LNK_RDY_LED color "red"
           }
		
		if {[expr $DATA_KR_04B1 & 0x00000100]} {
             dashboard_set_property $dash SEQ_RECONFG_MODE text "AN mode"
           } elseif {[expr $DATA_KR_04B1 & 0x00000200]} {
             dashboard_set_property $dash SEQ_RECONFG_MODE text "LT mode"
           } elseif {[expr $DATA_KR_04B1 & 0x00000400]} {
             dashboard_set_property $dash SEQ_RECONFG_MODE text "10G Data Mode"
           } elseif {[expr $DATA_KR_04B1 & 0x00002000]} {
             dashboard_set_property $dash SEQ_RECONFG_MODE text "10G FEC Mode"
           } else {
		   dashboard_set_property $dash SEQ_RECONFG_MODE text "---"
           }
		if {[expr $DATA_KR_04B1 & 0x00002000]} {
			  dashboard_set_property $dash fec_use_data color "green"
           } else {
             dashboard_set_property $dash fec_use_data color "red"
           }

			
	set DATA_KR_04C0 [reg_read [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR]]
	#puts "0x04C0 $DATA_KR_04C0"
	    if {[expr $DATA_KR_04C0 & 0x00000001]} {
             dashboard_set_property $dash an_ena_data color "green"
           } else {
             dashboard_set_property $dash an_ena_data color "red"
           }
		   
	set DATA_KR_04C2 [reg_read [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04C2] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR]]
	#puts "0x04C2 $DATA_KR_04C2"
	    if {[expr $DATA_KR_04C2 & 0x00000004]} {
             dashboard_set_property $dash an_comp_data text "Complete"
           } else {
             dashboard_set_property $dash an_comp_data text "In Progress"
           }
		   
		if {[expr $DATA_KR_04C2 & 0x00000020]} {
             dashboard_set_property $dash an_abi_data color "green"
           } else {
             dashboard_set_property $dash an_abi_data color "red"
           }
		   
	    if {[expr $DATA_KR_04C2 & 0x00000040]} {
             dashboard_set_property $dash an_stat_data color "green"
           } else {
             dashboard_set_property $dash an_stat_data color "red"
           }
		if {[expr $DATA_KR_04C2 & 0x00000080]} {
             dashboard_set_property $dash lp_an_data color "green"
           } else {
             dashboard_set_property $dash lp_an_data color "red"
           }   
		   
		if {[expr $DATA_KR_04C2 & 0x00000200]} {
             dashboard_set_property $dash seq_an_data text "Failure detected"
           } else {
             dashboard_set_property $dash seq_an_data text "Failure not detected"
           }
		   
		if {[expr $DATA_KR_04C2 & 0x00001000]} {
             dashboard_set_property $dash fi_an_data text "1000BASE-KX"
           } elseif {[expr $DATA_KR_04C2 & 0x00002000]} {
             dashboard_set_property $dash fi_an_data text "10GBASE-KX4"
           } elseif {[expr $DATA_KR_04C2 & 0x00004000]} {
             dashboard_set_property $dash fi_an_data text "10GBASE-KR"
           } elseif {[expr $DATA_KR_04C2 & 0x00008000]} {
             dashboard_set_property $dash fi_an_data text "40GBASE-KR4"
           } elseif {[expr $DATA_KR_04C2 & 0x00010000]} {
             dashboard_set_property $dash fi_an_data text "40GBASE-CR4"
           } elseif {[expr $DATA_KR_04C2 & 0x00020000]} {
             dashboard_set_property $dash fi_an_data text "100GBASE-CR10"
           } else {
			 dashboard_set_property $dash fi_an_data text "---"
		   }

	proc_lt_all_status
	
	if { [ string match [dashboard_get_property $dash enable_design_example checked] "true" ] & [ string match [dashboard_get_property $dash enable_serial_loopback checked] "true" ] } {
	proc_xgmii_status
	}
	
	if { [dashboard_get_property $dash enable_design_example checked] } {
		
			proc_xgmii_packet_status
	}
	
	
}

proc proc_lt_all_status {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	
	set DATA_KR_04D0 [reg_read [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04D0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] ]
	#puts "0x04D0 $DATA_KR_04D0"
	    if {[expr $DATA_KR_04D0 & 0x00000001]} {
             dashboard_set_property $dash lt_ena_data color "green"
           } else {
             dashboard_set_property $dash lt_ena_data color "red"
           }
	
	set DATA_KR_04D2 [reg_read 0x4D2 0x0800]
		#if {[expr $DATA_KR_04D2 & 0x00000002]} {
         #    dashboard_set_property $dash lt_frame_lock_data color "green"
          # } else {
           #  dashboard_set_property $dash lt_frame_lock_data color "red"
           #}
		   
	set DATA_KR_04D4 [reg_read 0x4D4 0x0800]
		#if {[expr $DATA_KR_04D4 & 0x40000000]} {
         #    dashboard_set_property $dash lt_link_partner_comp_data color "green"
          # } else {
           #  dashboard_set_property $dash lt_link_partner_comp_data color "red"
           #}
	
	proc_lt_status {0}

}

proc proc_lt_status {chan} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	global htile_prod_enabled
	
	#set chan 0
	#set chan 0
	if {$htile_prod_enabled} {
	reconfig_reg_write $chan 0x542 0x0 0x0
	}
	
	set vod [reconfig_reg_read $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_VOD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]]
	set vod [expr $vod & 0x1f]
	
	set post [reconfig_reg_read $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_POST] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]]
	if {$post & 0x40} {
        set post [expr -($post & 0x1f)]
    } else {
        set post [expr $post & 0x1f]
    }
	
	set pre [reconfig_reg_read $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_PRE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]]
    if {$pre & 0x20} {
        set pre [expr -($pre & 0x1f)]
    } else {
        set pre [expr $pre & 0x1f]
    }
	dashboard_set_property $dash conv_tx_l$chan text "$vod, $post, $pre"
	
	#  Test bus = ADP 171[4:1]=1
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPBLOCK] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x1E 0x16
    
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 5
	reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
	set vga [expr [ reconfig_reg_read $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x1F]
	dashboard_set_property $dash conv_vga_l$chan text "$vga"
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 4
	reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    set ctle_dc [expr [ reconfig_reg_read $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x3F]
	dashboard_set_property $dash conv_ctledc_l$chan text "$ctle_dc"
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 3
	reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    set ctle_ac [expr [expr [ reconfig_reg_read $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] >> 3] & 0xF]
	dashboard_set_property $dash conv_ctleac_l$chan text "$ctle_ac"
	
	#  Test bus = ADP 171[4:1]=1
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPBLOCK] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x1E 0x16
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 7
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    set tap1 [expr [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x7F]
    if {$tap1 & 0x40} {
        set tap1 [expr -($tap1 & 0x3f)]
    } else {
        set tap1 [expr $tap1 & 0x3f]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 8
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    set tap2 [expr [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x3F]
    if {$tap2 & 0x20} {
         set tap2 [expr -($tap2 & 0x1f)]
    } else {
       
		set tap2 [expr $tap2 & 0x1f]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 9
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    set tap3 [expr [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x3F]
    if {$tap3 & 0x20} {
        
		set tap3 [expr -($tap3 & 0x1f)]
    } else {
        set tap3 [expr $tap3 & 0x1f]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 10
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    set tap4 [expr [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x1F]
    if {$tap4 & 0x10} {
        set tap4 [expr -($tap4 & 0xf)]
    } else {
        
		set tap4 [expr $tap4 & 0xf]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 11
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    set tap5 [expr [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x1F]
    if {$tap5 & 0x10} {
         set tap5 [expr -($tap5 & 0xf)]
    } else {
       
		set tap5 [expr $tap5 & 0xf]
    }
   
    reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 12
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
    set tap6 [expr [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x1F]
    if {$tap6 & 0x10} {
        set tap6 [expr -($tap6 & 0xf)]
    } else {
        
		set tap6 [expr $tap6 & 0xf]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 13
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
	set tap7 [expr [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]] & 0x1F]
	if {$tap7 & 0x10} {
        set tap7 [expr -($tap7 & 0xf)]
    } else {
        
		set tap7 [expr $tap7 & 0xf]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 14
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
	set tap8_tap9 [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]]
	set tap8 [expr $tap8_tap9 & 0xF]
	if {$tap8 & 0x4} {
        set tap8 [expr -($tap8 & 0x3)]
    } else {
        
		set tap8 [expr $tap8 & 0x3]
    }
	set tap9 [expr [expr $tap8_tap9 >> 3] & 0xF]
	if {$tap9 & 0x8} {
        set tap9 [expr -($tap9 & 0x7)]
    } else {
        
		set tap9 [expr $tap9 & 0x7]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 15
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
	set tap10_tap11 [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]]
	set tap10 [expr $tap10_tap11 & 0xF]
	if {$tap10 & 0x8} {
        set tap10 [expr -($tap10 & 0x7)]
    } else {
        
		set tap10 [expr $tap10 & 0x7]
    }
	set tap11 [expr [expr $tap10_tap11 >> 4] & 0xF]
	if {$tap11 & 0x8} {
        set tap11 [expr -($tap11 & 0x7)]
    } else {
        
		set tap11 [expr $tap11 & 0x7]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 16
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
	set tap12_tap13 [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]]
	set tap12 [expr $tap12_tap13 & 0xF]
	if {$tap12 & 0x8} {
        set tap12 [expr -($tap12 & 0x7)]
    } else {
        
		set tap12 [expr $tap12 & 0x7]
    }
	set tap13 [expr [expr $tap12_tap13 >> 4] & 0xF]
	if {$tap13 & 0x8} {
        set tap13 [expr -($tap13 & 0x7)]
    } else {
        
		set tap13 [expr $tap13 & 0x7]
    }
	
	reconfig_reg_rmw $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPWRITE] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x3F 17
    reconfig_reg_write $chan [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0
	set tap14_tap15 [reconfig_reg_read $chan   [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_LT_ADAPREAD] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO]]
	set tap14 [expr $tap14_tap15 & 0xF]
	if {$tap14 & 0x8} {
        set tap14 [expr -($tap14 & 0x7)]
    } else {
        
		set tap14 [expr $tap14 & 0x7]
    }
	set tap15 [expr [expr $tap14_tap15 >> 4] & 0xF]
	if {$tap15 & 0x8} {
        set tap15 [expr -($tap15 & 0x7)]
    } else {
        
		set tap15 [expr $tap15 & 0x7]
    }
	
	dashboard_set_property $dash conv_dfe_l$chan text "$tap1, $tap2, $tap3, $tap4, $tap5, $tap6, $tap7, $tap8, $tap9, $tap10, $tap11, $tap12, $tap13, $tap14, $tap15"
	
	if {$htile_prod_enabled} {
	reconfig_reg_write $chan 0x542 0x0 0x1
	}
}

proc proc_read_fec_status {} {

	global dash
	
	set reg_0xE4 [reg_read 0xE4]
	if {[expr $reg_0xE4 & 0x10] } {
	dashboard_set_property $dash fec_word_align_data color green
	} else {
	dashboard_set_property $dash fec_word_align_data color red
	}
	
	set reg_0xDC [reg_read 0xDC]
	set reg_0xDD [reg_read 0xDD]
	set reg_0xDE [reg_read 0xDE]
	set reg_0xDF [reg_read 0xDF]
	
	set temp0_corrected_blk [split [concat [format %0.8b $reg_0xDF] [format %0.8b $reg_0xDE] [format %0.8b $reg_0xDD] [format %0.8b $reg_0xDC]  ""] " "]
	set temp1_corrected_blk [join $temp0_corrected_blk ""]
	set corrected_codewords [bin2dec $temp1_corrected_blk]
	
	dashboard_set_property $dash corrected_codewords_data text $corrected_codewords
		
	set reg_0xE0 [reg_read 0xE0]
	set reg_0xE1 [reg_read 0xE1]
	set reg_0xE2 [reg_read 0xE2]
	set reg_0xE3 [reg_read 0xE3]
	
	set temp0_uncorrected_blk [split [concat [format %0.8b $reg_0xE3] [format %0.8b $reg_0xE2] [format %0.8b $reg_0xE1] [format %0.8b $reg_0xE0]  ""] " "]
	set temp1_uncorrected_blk [join $temp0_uncorrected_blk ""]
	set uncorrected_codewords [bin2dec $temp1_uncorrected_blk]
	
	dashboard_set_property $dash uncorrected_codewords_data text $uncorrected_codewords
	
}

proc bin2dec bin {
		if {$bin == 0} {
			return 0
		} elseif {[string match -* $bin]} {
			set sign -
			set bin [string range $bin[set bin {}] 1 end]
		} else {
			set sign {}
		}
		return $sign[expr 0b$bin]
	}
	
proc proc_reset_seq {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	
	#reconfig_reg_rmw {chan addr offset mask val}
	reconfig_reg_rmw 0 [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x1 0x1

}

proc proc_seq_force {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	global val
	
	if {[string match [dashboard_get_property $dash seq_force_mode selectedItem] "No Force"]} {
	
		#enable AN
		reg_rmw 0x4C0 0x0800 0x1 0x1
		#enable LT
		reg_rmw 0x4D0 0x0800 0x1 0x1
		#reset KR sequencer
		reg_rmw 0x4B0 0x0800 0x1 0x1
	
	reconfig_reg_rmw 0 [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x01 0x01
	} elseif {[string match [dashboard_get_property $dash seq_force_mode selectedItem] "10GBASE-R"]} {
	#set val 0x41
		#Disable AN
		reg_rmw 0x4C0 0x0800 0x1 0x0
		#Disable LT
		reg_rmw 0x4D0 0x0800 0x1 0x0		
		#reset KR sequencer
		reg_rmw 0x4B0 0x0800 0x1 0x1
	reconfig_reg_rmw 0 [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x000000f1 0x41
	} elseif {[string match [dashboard_get_property $dash seq_force_mode selectedItem] "10GBASE-KR"]} {
	
	#set val 0x51
		#enable AN
		reg_rmw 0x4C0 0x0800 0x1 0x1
		#enable LT
		reg_rmw 0x4D0 0x0800 0x1 0x1
		#reset KR sequencer
		reg_rmw 0x4B0 0x0800 0x1 0x1
		
	reconfig_reg_rmw 0 [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x000000f1 0x51
	} elseif {[string match [dashboard_get_property $dash seq_force_mode selectedItem] "10GBASE-KR FEC"]} {
	#set val 0xc1
		#enable AN
		reg_rmw 0x4C0 0x0800 0x1 0x1
		#enable LT
		reg_rmw 0x4D0 0x0800 0x1 0x1
		#reset KR sequencer
		reg_rmw 0x4B0 0x0800 0x1 0x1
	reconfig_reg_rmw 0 [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x000000f1 0xc1
	}

	#reconfig_reg_rmw {chan addr offset mask val}
	#reconfig_reg_rmw 0 [dict get $MAIN_DICT_s10_10g_kr_0 ADDR_KR_04B0] [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0xffffff0e $val

}

proc proc_enable_serial_loopback {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	
	if {[dashboard_get_property $dash enable_serial_loopback checked]} {
		reconfig_reg_rmw 0 0x4e1 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x01 0x1
		reconfig_reg_rmw 0 0x4e1 0x1000 0x01 0x1
		if {[dashboard_get_property $dash enable_design_example checked]} {
				
		dashboard_set_property $dash XGMII_Status visible true
		
		}
	} else {
		reconfig_reg_rmw 0 0x4e1 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10RECO] 0x01 0x0
		reconfig_reg_rmw 0 0x4e1 0x1000 0x01 0x0
		
		if {[dashboard_get_property $dash enable_design_example checked]} {
			
			#dashboard_remove $dash XGMII_Status
			dashboard_set_property $dash XGMII_Status visible false
		}
	}	
		
}

proc proc_disable_lf_timer {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	
	if {[dashboard_get_property $dash disable_lf_timer checked]} {
		reconfig_reg_rmw 0 0x4b0 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x04 0x04
	} else {
		reconfig_reg_rmw 0 0x4b0 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x04 0x00
	}	
		
}

proc proc_enable_tx_tuning {} {
	
	global dash
	global MAIN_DICT_s10_10g_kr_0
	
	if {[dashboard_get_property $dash enable_tx_tuning checked]} {
		reconfig_reg_rmw 0 0x4d0 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x04 0x04
		reconfig_reg_rmw 0 0x4d0 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x08 0x08
	} else {
		reconfig_reg_rmw 0 0x4d0 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x04 0x0
		reconfig_reg_rmw 0 0x4d0 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x08 0x0
	}	
	
}

proc proc_ignore_nonce {} {
	
	global dash
	global MAIN_DICT_s10_10g_kr_0
	
	if {[dashboard_get_property $dash ignore_tx_nonce checked]} {
		reconfig_reg_rmw 0 0x4c0 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x80 0x80
		reconfig_reg_rmw 0 0x4c0 0x1800 0x80 0x80
		
	} else {
		reconfig_reg_rmw 0 0x4c0 [dict get $MAIN_DICT_s10_10g_kr_0 OFFSET_S10CSR] 0x80 0x00
		reconfig_reg_rmw 0 0x4c0 0x1800 0x80 0x00
	}	
	
	
}

proc proc_enable_design_example {} {

	global dash
	
	if {[dashboard_get_property $dash enable_design_example checked]} {
				
		dashboard_set_property $dash start_xgmii_packets enabled true
		dashboard_set_property $dash clear_xgmii_packets enabled true
		
		if {[dashboard_get_property $dash enable_serial_loopback checked]} {
	
		dashboard_set_property $dash XGMII_Status visible true
	
	}
	
	} else {
		dashboard_set_property $dash start_xgmii_packets enabled false
		dashboard_set_property $dash clear_xgmii_packets enabled false
		
		if {[dashboard_get_property $dash enable_serial_loopback checked]} {
			
			#dashboard_remove $dash XGMII_Status
			dashboard_set_property $dash XGMII_Status visible false
		}
	}
	
	
	
}

proc proc_xgmii_packet_status {} {

	global dash
	
	global accum_send_packets
	
	set DATA_KR_04 [reg_read 0x04 0xf000]
	
	set accum_send_packets [ expr $accum_send_packets + $DATA_KR_04 ]
	dashboard_set_property $dash xgmii_received_packets_data text "$accum_send_packets"
	

}


proc proc_xgmii_status {} {

	global dash
	
	
	set DATA_KR_02 [reg_read 0x02 0xf000]
		if {[expr $DATA_KR_02 & 0x00000010]} {
             dashboard_set_property $dash xgmii_test_done_data color "green"
           } else {
             dashboard_set_property $dash xgmii_test_done_data color "red"
           }
		   
		 if {[expr $DATA_KR_02 & 0x00000001]} {
             dashboard_set_property $dash xgmii_rx_ready_data color "green"
           } else {
             dashboard_set_property $dash xgmii_rx_ready_data color "red"
           }  
		
		if {[expr $DATA_KR_02 & 0x00000002]} {
             dashboard_set_property $dash xgmii_checker_pass_data color "green"
           } else {
             dashboard_set_property $dash xgmii_checker_pass_data color "red"
           } 	
		   
		if {[expr $DATA_KR_02 & 0x00000004]} {
             dashboard_set_property $dash xgmii_rx_mismatch_data color "red"
           } else {
             dashboard_set_property $dash xgmii_rx_mismatch_data color "green"
           } 	

		if {[expr $DATA_KR_02 & 0x00000040]} {
             dashboard_set_property $dash xgmii_test_pass_data color "green"
           } else {
             dashboard_set_property $dash xgmii_test_pass_data color "red"
           }	
}

proc proc_start_xgmii_packets {} {
	
	global dash
	
	#reconfig_reg_rmw 0 0x00 0xf000 0x01 0x01
	reg_write 0xf000 0x00 0x1
	
}

proc proc_clear_xgmii_packets {} {
		global dash
		global accum_send_packets
		
		set accum_send_packets 0
		dashboard_set_property $dash xgmii_received_packets_data text "0"
}

proc proc_fec_insert_error {} {

	global dash
	
	reconfig_reg_rmw 0 0x4b2 0x0800 0x800 0x800
	
}

proc proc_trigger_adapt {} {
	global dash
	global iadpt_set

	if {[string match [dashboard_get_property $dash trigger_adapt_button text] "Start Adaptation"]} {
		dashboard_set_property $dash trigger_adapt_button text "Stop Adaptation"
		set iadpt_set 1
		proc_trigger_adapt_ll10g
		
	} elseif {[string match [dashboard_get_property $dash trigger_adapt_button text] "Stop Adaptation"]} {
		dashboard_set_property $dash trigger_adapt_button text "Start Adaptation"
		set iadpt_set 0
		proc_trigger_adapt_ll10g
	}
	
	

}
proc proc_trigger_adapt_ll10g {} {
	global ip_sub_variant_option
	
	iadpt_control 0x0000
		
	
}
proc dashboard_s10_10g_kr_0 {} {
	global dash
	global MAIN_DICT_s10_10g_kr_0
	global rs_enabled
	global htile_prod_enabled
	
	#set dash [add_service dashboard Ethernet_Link_Monitor_s10_10g_kr_0 "Ethernet Link Monitor" "Tools/Ethernet_Link_Monitor_s10_10g_kr_0"]
	#dashboard_set_property $dash self visible true
	
	if {$rs_enabled} {	
		widget_group 1  topGroup ip_tab 1 "Stratix 10 10GBASE-KR PHY IP w/Firecode FEC"
	} else {
		widget_group 1  topGroup ip_tab 1 "Stratix 10 10GBASE-KR PHY IP"
	}
	
		#widget_label 1  ip_var topGroup "IP Variant: Stratix 10 10GBASE-KR PHY IP"
		widget_checkBox 1  continuous_read topGroup "Continuous Read All Registers" proc_continuous_read
	
		dashboard_add $dash allstatus tabbedGroup topGroup
	
			#
			#PHY - KR
			#
			widget_group 1  KR4 allstatus 1 "KR"
			
				widget_button 1  read_kr4_status KR4 "Read KR Status" proc_read_kr4_status				
				widget_checkBox 1 enable_serial_loopback KR4 "Serial Loopback" proc_enable_serial_loopback
				
				widget_group 1  kr4_dummy_1 KR4 3 ""
		
				#
				#Sequencer
				#
				widget_group 1  SEQ kr4_dummy_1 3 "Sequencer"
					
					widget_button 1  reset_seq SEQ "Reset Sequencer"  proc_reset_seq
					
					
					widget_checkBox 1 disable_lf_timer SEQ "Disable LF Timer" proc_disable_lf_timer
					widget_label 1   seq_blank_2 SEQ ""
		
					widget_label 1   seq_force SEQ "SEQ Force Mode"
					widget_comboBox 1  seq_force_mode SEQ {"No Force" "10GBASE-R" "10GBASE-KR"  "10GBASE-KR FEC"}
					widget_button 1  seq_force SEQ "Force" proc_seq_force
					
					widget_label 1  SEQ_LNK_RDY SEQ "SEQ Link Ready"
					widget_led 1  SEQ_LNK_RDY_LED SEQ ""  black
					widget_label 1   seq_blank_3 SEQ ""
		
					widget_label 1  SEQ_MODE SEQ "SEQ Mode"
					widget_textBox 1  SEQ_RECONFG_MODE SEQ 0 "---" 7 125 
					widget_label 1   seq_blank_3 SEQ ""
					
					widget_groupHeight 1 seq_status_dummygroup SEQ 1 "" 100
				#
				# Auto Negotiation & FEC Parameters
				#
				widget_group 1  AN_FEC kr4_dummy_1 2 "Auto Negotiation & FEC"
					
					widget_checkBox 1 ignore_tx_nonce AN_FEC "Ignore Nonce Field" proc_ignore_nonce
					widget_label 1  ignore_tx_nonce_dummy AN_FEC ""
					
					widget_group 1  AN AN_FEC 2 "AN"
						widget_label 1  an_ena_title AN "AN Enable"
						widget_led 1  an_ena_data AN ""  black
						widget_label 1  an_comp_title AN "AN Complete"
						widget_label 1  an_comp_data AN ""
						widget_label 1  an_abi_title AN "AN Ability"
						widget_led 1  an_abi_data AN "" black
						widget_label 1  an_stat_title AN "AN Status"
						widget_led 1 	an_stat_data AN ""  black
						#widget_label 1  lp_an_abi_title AN "LP AN Ability"
						#widget_label 1  lp_an_abi_data AN ""
						widget_label 1  seq_an_title AN "SEQ AN"
						widget_textBox 1  seq_an_data AN 0 "---" 7 125
						widget_label 1  fi_an_title AN "Final AN Tech"
						widget_textBox 1 fi_an_data AN 0 "---" 7 125
						widget_label 1  lp_an_title AN "LP AN Ability"
						widget_led 1 	lp_an_data AN "" black
					
					widget_group 1  FEC AN_FEC 2 "FEC"
						
						widget_button 1  fec_insert_error FEC "FEC Insert Error" proc_fec_insert_error
						widget_label 1 	fec_insert_error_dummy FEC ""
						
						widget_label 1  fec_ena_title FEC "FEC Enable"
						widget_led 1 	fec_ena_data FEC ""  black
						
						widget_label 1  fec_req_title FEC "FEC Request Enable"
						widget_led 1  fec_req_data FEC ""  black
						
						widget_label 1  fec_use_title FEC "FEC in use"
						widget_led 1  fec_use_data FEC ""  black
						
						
						widget_groupHeight 1 fec_status_dummygroup FEC 1 "" 50
						#widget_label 1  fec_abi_title FEC "FEC ability"
						#widget_led 1 	fec_abi_data FEC ""  black
				
				if {$rs_enabled} {
				widget_group 1  FEC_Stats kr4_dummy_1 2 "FEC Statistics"
					
					widget_button 1  read_fec_status FEC_Stats "Read FEC Status" proc_read_fec_status
					widget_label 1  dummy_fec_stats FEC_Stats ""
					
					widget_label 1  fec_word_align_title FEC_Stats "FEC Word Align"
					widget_led 1  fec_word_align_data FEC_Stats ""  black	
					
					widget_label 1  corrected_codewords_label FEC_Stats "Corrected Blocks"
					widget_textBox 1  corrected_codewords_data FEC_Stats  0 "" 7 100
					
					widget_label 1  uncorrected_codewords_label FEC_Stats "Uncorrected Blocks"
					widget_textBox 1  uncorrected_codewords_data FEC_Stats  0 "" 7 100
					
				dashboard_set_property $dash FEC_Stats visible false	
				}
				
				#
				# Link Training Parameters
				#
				widget_group 1  LT_XGMII KR4 2 "Link Training"
				
					widget_group 1  LT LT_XGMII 2 "Link Training"
										
						widget_checkBox 1 enable_tx_tuning LT "Enable TX Tuning" proc_enable_tx_tuning
						dashboard_set_property $dash enable_tx_tuning checked true
						widget_button 1 trigger_adapt_button LT "Stop Adaptation"  proc_trigger_adapt
						
						widget_checkBox 1  htile_prod LT "H-Tile Production Device" proc_htile_prod_enable
						set	htile_prod_enabled 0
						widget_label 1  lt_init LT ""
						
						widget_label 1  lt_ena_title LT "LT Enable"
						widget_led 1  lt_ena_data LT ""  black
						
						#widget_label 1  lt_frame_lock_title LT "LT Frame Lock"
						#widget_led 1  lt_frame_lock_data LT ""  black
						
						#widget_label 1  lt_link_partner_comp_title LT "Link Partner LT complete"
						#widget_led 1  lt_link_partner_comp_data LT ""  black
												
						widget_label 1  conv_tx_title LT "Converged TX (vod, post, pre)"
						widget_label 1  conv_tx_l0 LT "0, 0, 0"
						
						widget_label 1  conv_vga_title LT "Converged VGA"
						widget_label 1  conv_vga_l0 LT "0"
						
						widget_label 1  conv_ctledc_title LT "Converged CTLE DC"
						widget_label 1  conv_ctledc_l0 LT "0"
											
						widget_label 1  conv_ctleac_title LT "Converged CTLE AC"
						widget_label 1  conv_ctleac_l0 LT "0"
						
						widget_label 1  conv_dfe_title LT "Converged DFE"
						widget_labelwidth 1  conv_dfe_l0 LT "0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0" 225
						
					widget_group 1  XGMII LT_XGMII 1 "XGMII"
							
						widget_checkBox 1 enable_design_example XGMII "Quartus Generated Design Example used" proc_enable_design_example
						
						widget_button 1 start_xgmii_packets XGMII "Send Packets" proc_start_xgmii_packets
						dashboard_set_property $dash start_xgmii_packets enabled false
						
						widget_label 1 note_1 XGMII "Send Packets triggers 418 packets"
						dashboard_set_property $dash note_1 foregroundColor blue
						
						widget_label 1 note_2 XGMII "During continuous read all registers, packets are sent continuously"
						dashboard_set_property $dash note_2 foregroundColor blue
						
						widget_group 1  XGMII_Packets XGMII 3 ""
						
						widget_label 1  xgmii_received_packets_label XGMII_Packets "XGMII Received Packets"
						widget_textBox 1  xgmii_received_packets_data XGMII_Packets  0 "" 7 100
						widget_button 1 clear_xgmii_packets XGMII_Packets "Clear" proc_clear_xgmii_packets
						dashboard_set_property $dash clear_xgmii_packets enabled false
						
						
						widget_group 1  XGMII_Status XGMII 2 ""
						
						widget_label 1  xgmii_test_done_label XGMII_Status "XGMII Test Done"
						widget_led 1  xgmii_test_done_data XGMII_Status ""  black
											
						widget_label 1  xgmii_rx_ready_label XGMII_Status "XGMII RX Ready"
						widget_led 1  xgmii_rx_ready_data XGMII_Status ""  black
											
						widget_label 1  xgmii_checker_pass_label XGMII_Status "XGMII Checker Pass"
						widget_led 1  xgmii_checker_pass_data XGMII_Status ""  black
												
						widget_label 1  xgmii_rx_mismatch_label XGMII_Status "XGMII RX No Mismatch"
						widget_led 1  xgmii_rx_mismatch_data XGMII_Status ""  black
												
						widget_label 1  xgmii_test_pass_label XGMII_Status "XGMII Test Pass"
						widget_led 1  xgmii_test_pass_data XGMII_Status ""  black		
												
						if { [ string match [dashboard_get_property $dash enable_design_example checked] "false" ] & [ string match [dashboard_get_property $dash enable_serial_loopback checked] "false" ] } {
						dashboard_set_property $dash XGMII_Status visible false
						}
						
			help_tab	
								
}						

proc help_tab {} {

	global dash

	widget_group 1  Help allstatus 1 "Help"		

		widget_group 1 instructions Help 1 "KR Link Bring Up Guidelines"
		
								
				widget_label 1 instructions_2 instructions	"if (Internal Serial Loop-back or External Loop-back) ("
					dashboard_set_property $dash instructions_2 foregroundColor blue			
				widget_label 1 select_ip_note instructions	"      1. Check Ignore Nonce check box"
					#dashboard_set_property $dash select_ip_note foregroundColor blue
				widget_label 1 instructions_1 instructions	"      2. Click Reset sequencer button"
					#dashboard_set_property $dash instructions_1 foregroundColor blue
				widget_label 1 instructions_2 instructions	"      if ( Internal Serial Loop-back) ("
					dashboard_set_property $dash instructions_2 foregroundColor blue
				widget_label 1 instructions_2a instructions	"            3. Check Serial Loop-back check box"
				widget_label 1 instructions_2b instructions	"      )"
					dashboard_set_property $dash instructions_2b foregroundColor blue														
				widget_label 1 instructions_2 instructions	"      4. Click on Continuous Read All registers or Read KR Status. The link should be up"
					#dashboard_set_property $dash instructions_2 foregroundColor blue	
				widget_label 1 instructions_Mis_Kr instructions	") elseif (Connect to Link Partner) ("
					dashboard_set_property $dash instructions_Mis_Kr foregroundColor blue
				widget_label 1 instructions_2 instructions	"      1. Click on Continuous Read All registers or Read KR Status"
				widget_label 1 instructions_3 instructions	")"
					dashboard_set_property $dash instructions_3 foregroundColor blue
				
				
		widget_groupHeight 1 help_status_dummygroup Help 1 "" 400
		
}						
#==============================================================================
#==============================================================================
#  Main program starts here 
#==============================================================================
#==============================================================================

  #design_load de_wrapper.sof
  de_script_intro
  de_script_help
  #puts "Use the \"de_script_help\"   command to see the above message"
  #puts " "
  #get_jtag_masters
  #puts "Use command \" get_jtag_masters \" to get list of available Jtag Masters"
  #puts "Use command \" open_jtag <number> \" to open Jtag Service"
  #puts "e.g. \" open_jtag 0 \" will open first Jtag returned by command \"get_jtag_masters \". "
  #open_jtag
  #rst_all



#==============================================================================
proc proc_create_dash {} {
	
	global dash
	
	widget_label 0 select_ip_option_note select_ip_option "Plotting link monitor user interface ... please wait!!"
	dashboard_set_property $dash select_ip_option_note foregroundColor red
	dashboard_s10_10g_kr_0
	widget_label 0 select_ip_option_note select_ip_option "Link monitor user interface ready to use!!"
	dashboard_set_property $dash select_ip_option_note foregroundColor blue

}